package org.one.stone.soup.mjdb.data.field;

import java.util.*;
/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

/**
	* The DateField class is a wrapper for a date value.
	* @author Nik Cross
*/
public class DateField extends Field{
	public static String DEFAULT_ID = "_Date";

	private int day;
	private int dayNo;

	private int month;
	public static final String[] SHORT_MONTH_NAME = new String[]{
		"Jan",
		"Feb",
		"Mar",
		"Apr",
		"May",
		"Jun",
		"Jul",
		"Aug",
		"Sep",
		"Oct",
		"Nov",
		"Dec"
	};
	private int monthNo;
	private int year;

	private char delimiter='/';
/**
 * This constructor creates a new DateField with a value of todays date.
 */

public DateField() {
	super();

	java.util.Calendar cal = java.util.Calendar.getInstance();
	day = cal.get(java.util.Calendar.DAY_OF_MONTH);
	month = cal.get(java.util.Calendar.MONTH)+1;
	year = cal.get(java.util.Calendar.YEAR);
	dayNo = cal.get(java.util.Calendar.DAY_OF_WEEK);
}
/**
 * This constructor sets the initial date value.
 * @param day	The day number in the month
 * @param month	The month number 1-12
 * @param year	The year number (yyyy)
 */

public DateField(int day,int month,int year) {
	super();

	this.day=day;
	this.month=month;
	this.year=year;

	Calendar cal = Calendar.getInstance();
	cal.set(year,month-1,day);
	dayNo = cal.get(Calendar.DAY_OF_WEEK);
}
/**
 * This constructor sets the initial date value.
 * Any delimiter character can be used.
 * Formats excepted:
 * DD?MM?YY
 * DD?MM?YYYY
 * @param date	The date string
 */

public DateField(String date) throws ConstructException{
	super();

	try{
		setValue(null,date,this);
	}
	catch(LockException le){}// dummy catch
}
/**
 * Add a number of days to the date.
 * Returns the resulting DateField.
 * @param days	The number of days to add
 */

public DateField addDays(int days) {

	Calendar cal = Calendar.getInstance();
	cal.set(year,month-1,day);

	cal.add(Calendar.DAY_OF_YEAR,days);

	return new DateField(cal.get(Calendar.DATE),cal.get(Calendar.MONTH)+1,cal.get(Calendar.YEAR));
}
/**
 * Compares this date with another date.
 * Returns true if this date is after the supplied date.
 * @param date		The date to compare with
 */

public boolean after(DateField date) {
	return after(date,false);
}
/**
 * Compares this date with another date.
 * If inclusive is true
 *  Returns true if this date is on or after the supplied date.
 * Else
 *  Returns true if this date is after the supplied date.
 * @param date		The date to compare with
 * @param inclusive	If true, a date on or after the date supplied returns true.
 */

public boolean after(DateField date,boolean inclusive) {

	if(inclusive)
	{
		if(asLong()>=date.asLong())
			return true;
		else
			return false;
	}
	else
	{
		if(asLong()>date.asLong())
			return true;
		else
			return false;
	}
}
/**
 * Compares this date with another date.
 * Returns a long value for comparison with other dates.
 */

public long asLong() {
	return Long.parseLong(getYear()+getMonth()+getDay());
}
/**
 * Compares this date with another date.
 * Returns true if this date is before the supplied date.
 * @param date		The date to compare with
 */

public boolean before(DateField date) {

	return before(date,false);
}
/**
 * Compares this date with another date.
 * If inclusive is true
 *  Returns true if this date is on or before the supplied date.
 * Else
 *  Returns true if this date is before the supplied date.
 * @param date		The date to compare with
 * @param inclusive	If true, a date on or before the date supplied returns true.
 */

public boolean before(DateField date,boolean inclusive) {

	if(inclusive)
	{
		if(asLong()<=date.asLong())
			return true;
		else
			return false;
	}
	else
	{
		if(asLong()<date.asLong())
			return true;
		else
			return false;
	}
}
/**
 * Compares this object with the target object.
 * Returns EQ if the two date values are the same.
 * Returns GT if the this date is after the target date.
 * Returns LT if the this date is before the target date.
 * @param target	The Field to compare with (Accepts DateField)
 */

public int compare(Field target) throws CompareException {

	if(target instanceof DateField)
	{
		if(isSame((DateField)target))
			return EQ;
		else if(after((DateField)target))
			return GT;
		else
			return LT;
	}

	throw new CompareException("Can't compare "+target+" with date");

}
/**
 * Returns a String containing the date in the format DD?MM?YYYY.
 * The delimiter default is /. It can be set by a call to setDelimiter,
 * and is set if the date is set from a formatted string.
 */

public String getConsiseDate() {

	return getDay()+delimiter+getMonth()+delimiter+getYear();
}

public Calendar getCalendar()
{
	Calendar calendar = Calendar.getInstance();

	calendar.set(Calendar.DAY_OF_MONTH,day);
	calendar.set(Calendar.MONTH,month);
	calendar.set(Calendar.YEAR,year);

	return calendar;
}

/**
  * Returns a two character String containing the day in month number.
  * If the day number is less than 10 then a 0 is prefixed to the
  * String.
  */
public String getDay() {
	if(day<10)
		return "0"+day;
	else
		return ""+day;
}
public int getDayInt()
{
		return day;
}
public int getDayOfWeek() {
	return dayNo;
}
/**
 *
 * Returns the delimiter character.
 */
public char getDelimiter() {
	return delimiter;
}
/**
  * Returns a two character String containing the  month number (1 to 12).
  * If the month number is less than 10 then a 0 is prefixed to the
  * String.
  */

public String getMonth() {
	if(month<10)
		return "0"+month;
	else
		return ""+month;
}

public int getMonthInt()
	{
			return month;
	}

public String getMonthName() {
	return SHORT_MONTH_NAME[month-1];
}

/**
  * Returns a String containing the date in a short format like
  * Mon 22 Feb
  */

 public String getShortDate() {
	String dayString="?";

	switch(dayNo)
	{
		case Calendar.MONDAY:
			dayString="Mon";
			break;
		case Calendar.TUESDAY:
			dayString="Tue";
			break;
		case Calendar.WEDNESDAY:
			dayString="Wed";
			break;
		case Calendar.THURSDAY:
			dayString="Thu";
			break;
		case Calendar.FRIDAY:
			dayString="Fri";
			break;
		case Calendar.SATURDAY:
			dayString="Sat";
			break;
		case Calendar.SUNDAY:
			dayString="Sun";
			break;
	}

	return dayString+" "+day+" "+SHORT_MONTH_NAME[month-1]+" "+year;//+" "+getYear();
}
/**
  * Returns a DateField for the first day of the week containing this
  * date.
  */

 public DateField getStartOfWeek() {

	Calendar cal = Calendar.getInstance();
	cal.set(year,month-1,day);

	while(cal.get(Calendar.DAY_OF_WEEK) != cal.getFirstDayOfWeek())
		cal.roll(Calendar.DATE,false);

	return new DateField(cal.get(Calendar.DATE),cal.get(Calendar.MONTH)+1,cal.get(Calendar.YEAR));
}
/**
 * Returns the date held in this field as a String in the format DD?MM?YYYY.
 * @param name	The name of the data to return (Ignored in DateField)
 */

public String getValue(String name) {
	return getConsiseDate();
}
/**
  * Returns a four character String containing the year number.
  */

public String getYear() {
	return ""+year;
}

public int getYearInt()
	{
			return year;
	}
/**
 * Compares this date with another date.
 * Returns true if this date is the same as the supplied date.
 * @param date		The date to compare with
 */

public boolean isSame(DateField date){

	if( (date).asLong()==asLong())
		return true;
	else
		return false;
}
/**
 * Sets the consise date format delimiter.
 * @param newDelimiter		The consise date format delimiter
 */
public void setDelimiter(char newDelimiter) {
	delimiter = newDelimiter;
}
/**
 * Set the date value.
 * Any delimiter character can be used.
 * Formats excepted:
 * DD?MM?YY
 * DD?MM?YYYY
 * @param date	The date string
 */

public void setValue(String name, String value, Object source) throws LockException,ConstructException {

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	try{
	  if(value.length()==10)
	  {
		// 0123456789
		// DD-MM-YYYY
		this.day=Integer.parseInt(value.substring(0,2));
		this.month=Integer.parseInt(value.substring(3,5));
		this.year=Integer.parseInt(value.substring(6,10));

		this.delimiter = value.charAt(2);
	  }
	  else if(value.length()==8)
	  {
		// 01234567
		// DD-MM-YY
		this.day=Integer.parseInt(value.substring(0,2));
		this.month=Integer.parseInt(value.substring(3,5));
		this.year=Integer.parseInt(value.substring(6,8))+2000;

		this.delimiter = value.charAt(2);
	  }
	  else
	  {
		throw new ConstructException("Construct value from "+value);
	  }
	}
	catch(Exception e)
	{
		throw new ConstructException("Construct value from "+value+": "+e);
	}

	Calendar cal = Calendar.getInstance();
	cal.set(year,month-1,day);
	dayNo = cal.get(Calendar.DAY_OF_WEEK);
}
public String toString() {
	return getConsiseDate();
}

/**
 * This constructor creates a new DateField with a value of todays date.
 */

public DateField(long time) {
	super();

	Date date = new Date(time);

	java.util.Calendar cal = java.util.Calendar.getInstance();
	cal.setTime(date);
	day = cal.get(java.util.Calendar.DAY_OF_MONTH);
	month = cal.get(java.util.Calendar.MONTH)+1;
	year = cal.get(java.util.Calendar.YEAR);
	dayNo = cal.get(java.util.Calendar.DAY_OF_WEEK);
}
}
