package org.one.stone.soup.mjdb.data.field;

/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

/**
	* The FloatField class is a wrapper for an single float value.
	* It can be set as nonZero if it is to be used as a divisor.
	* @author Nik Cross
*/

public class FloatField extends Field {
	public static String DEFAULT_ID = "_Float";

	private double data;
	private boolean nonZero = false;
	private double valueMax= FLOAT_MAX_POS;
	private double valueMin = FLOAT_MAX_NEG;
/**
 * This constructor creates a new FloatField with a value of 0 and
 * the nonZero flag set to false.
 */

public FloatField() {
	super();
}
/**
 * This constructor sets the initial float value.
 * The nonZero flag is set to false.
 * @param data	The float value
 */

public FloatField(double data) {
	super();

	this.data=data;
}
/**
 * Compares this object with the target object.
 * Returns EQ if the two float values are the same.
 * Returns GT if the the target float value is greater than the target float value.
 * Returns LT if the the target float value is less than the target float value.
 * @param target	The Field to compare with (Accepts FloatField,IntField)
 */

public int compare(Field target) throws CompareException {

	if(target instanceof FloatField)
	{
		if( ((FloatField)target).getValue() == this.data)
			return EQ;
		else if( ((FloatField)target).getValue() > this.data)
			return GT;
		else
			return LT;
	}
	else if(target instanceof IntField)
	{
		if( ((IntField)target).getValue() == this.data)
			return EQ;
		else if( ((IntField)target).getValue() > this.data)
			return GT;
		else
			return LT;
	}

	throw new CompareException("Can't compare "+target+" with float");

}
/**
 * Returns the float data held in this field.
 */

public double getValue() {
	return data;
}
/**
 * Returns the float data held in this field, rounded to <code>dp</code>
 * decimal places (Rounded down).
 * @param dp	The number of decimal places to return
 */

public double getValue(int dp) {
	double div = Math.pow(10,dp);
	double add = Math.pow(10,-dp)*50;

	return Math.floor(data*div+add)/div;
}
/**
 * Returns the float data held in this field as a String.
 * @param name	The name of the field to return (Ignored in FloatField)
 */
public String getValue(String name) {
	return ""+data;
}
/**
 * Returns the maximum float value for this field.
 */

public double getValueMax() {
	return valueMax;
}
/**
 * Returns the minimum float value for this field.
 */

public double getValueMin() {
	return valueMin;
}
/**
 * Set the value of the nonZero flag.
 * If the nonZero flag is true, then the float value can not be set
 * to zero.
 * @param nonZero	The flag state
 */

public void setNonZero(boolean nonZero) {
	this.nonZero = nonZero;
}
/**
 * Set the minimum and maximum values for this float value.
 * If the nonZero flag is true, then the float value can not be set
 * to zero.
 * @param newMin	The minimum float value
 * @param newMax	The maximum float value
 * @param nonZero	The nonZero flag state
 */

public void setRange(double newMin,double newMax,boolean nonZero) {
	valueMin = newMin;
	valueMax = newMax;
	this.nonZero = nonZero;
}
/**
 * Set the float value.
 * @param newData		The new float value
 * @param source		The object requesting the value change
 */
public void setValue(float newData,Object source) throws LockException {

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	data = newData;

	if(report)
		reportChange(source);
}
/**
 * Set the float value.
 * @param name		The target field name (Ignored in FloatField)
 * @param value		A String containing the new float value
 * @param source	The object requesting the value change
 */

public void setValue(String name, String value, Object source) throws LockException
{
	setValue( new Double(value).floatValue(),source );
}
public String toString() {
	return Float.toString((float)data);
}
}
